@@ -510,7 +510,7 @@ static int incbeta_cfe (double x, double a, double b, double *result)
    double last_f;
    double factor;
 
-   factor = a*log(x)+b*log(1.0-x);
+   /* factor = a*log(x)+b*log(1.0-x); */
    factor = a*log(x)+b*log1p(-x);
    factor += JDMlog_gamma(a+b)-JDMlog_gamma(a) - JDMlog_gamma(b);
    factor = exp (factor)/a;
@@ -806,7 +806,7 @@ static double f_cdf_intrin (double *t, double *nu1p, double *nu2p)
 # define nc_median_ushorts nc_median_uints
 #endif
 
-#define GENERIC_TYPE char
+#define GENERIC_TYPE signed char
 #define MEAN_FUNCTION mean_chars
 #define MEAN_RESULT_TYPE float
 #define STDDEV_FUNCTION stddev_chars
