@@ -1,6 +1,6 @@
 /* file intrinsics for S-Lang */
 /*
-Copyright (C) 2004-2014 John E. Davis
+Copyright (C) 2004-2016 John E. Davis
 
 This file is part of the S-Lang Library.
 
@@ -46,9 +46,10 @@ USA.
 
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
-#endif
-#ifdef HAVE_SYS_FCNTL_H
-# include <sys/fcntl.h>
+#else
+# ifdef HAVE_SYS_FCNTL_H
+#  include <sys/fcntl.h>
+# endif
 #endif
 
 #ifdef __unix__
@@ -74,6 +75,10 @@ USA.
 # include <sys/stat.h>
 #endif
 
+#ifdef HAVE_SYS_STATVFS_H
+# include <sys/statvfs.h>
+#endif
+
 #if defined(VMS)
 # define USE_LISTDIR_INTRINSIC	0
 #else
@@ -122,6 +127,11 @@ USA.
 #include "slang.h"
 #include "_slang.h"
 
+#define SLSYSWRAP_READDIR readdir
+#ifdef SLSYSWRAP
+# include <slsyswrap.h>
+#endif
+
 static int is_interrupt (int e)
 {
 #ifdef EINTR
@@ -247,6 +257,89 @@ static void lstat_cmd (char *file)
 #endif				       /* HAVE_LSTAT */
 }
 
+#if defined(HAVE_STATVFS)
+static SLang_CStruct_Field_Type StatVFS_Struct [] =
+{
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_bsize, "f_bsize", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_bsize, "f_bsize", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_frsize, "f_frsize", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_blocks, "f_blocks", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_bfree, "f_bfree", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_bavail, "f_bavail", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_files, "f_files", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_ffree, "f_ffree", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_favail, "f_favail", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_fsid, "f_fsid", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_flag, "f_flag", 0),
+   MAKE_CSTRUCT_UINT_FIELD(struct statvfs, f_namemax, "f_namemax", 0),
+   SLANG_END_CSTRUCT_TABLE
+};
+
+static void statvfs_cmd (void)
+{
+   struct statvfs vfs;
+   FILE *fp;
+   char *path = NULL;
+   SLFile_FD_Type *f = NULL;
+   SLang_MMT_Type *mmt = NULL;
+   int fd, status;
+
+   switch (SLang_peek_at_stack ())
+     {
+      default:
+      case SLANG_INT_TYPE:
+	if (-1 == SLang_pop_int (&fd))
+	  return;
+	break;
+
+      case SLANG_FILE_FD_TYPE:
+	if (-1 == SLfile_pop_fd (&f))
+	  return;
+	if (-1 == SLfile_get_fd (f, &fd))
+	  goto free_and_return;
+	break;
+
+      case SLANG_FILE_PTR_TYPE:
+	if (-1 == SLang_pop_fileptr (&mmt, &fp))
+	  return;
+	fd = fileno (fp);
+	break;
+
+      case SLANG_STRING_TYPE:
+	if (-1 == SLang_pop_slstring (&path))
+	  return;
+	break;
+     }
+
+   if (path != NULL)
+     {
+	while ((-1 == (status = statvfs (path, &vfs)))
+	       && (is_interrupt (errno)))
+	  continue;
+     }
+   else
+     {
+	while ((-1 == (status = fstatvfs (fd, &vfs)))
+	       && (is_interrupt (errno)))
+	  continue;
+     }
+
+   if (status == 0)
+     (void) SLang_push_cstruct ((VOID_STAR) &vfs, StatVFS_Struct);
+   else
+     {
+	_pSLerrno_errno = errno;
+	(void) SLang_push_null ();
+     }
+
+   /* drop */
+free_and_return:
+   if (f != NULL) SLfile_free_fd (f);
+   if (mmt != NULL) SLang_free_mmt (mmt);
+   if (path != NULL) SLang_free_slstring (path);
+}
+#endif				       /* HAVE_STATVFS */
+
 #if defined(HAVE_UTIME) || defined(HAVE_UTIMES)
 static int utime_intrin (char *file, double *t0p, double *t1p)
 {
@@ -925,11 +1018,23 @@ static int build_dirlist (char *dir, char *opt, char ***listp, unsigned int *num
 
    num_files = max_num_files = 0;
    list = NULL;
-   while (NULL != (ep = readdir (dp)))
+   while (1)
      {
 	unsigned int len;
 	char *name;
 
+	errno = 0;
+	if (NULL == (ep = SLSYSWRAP_READDIR (dp)))
+	  {
+	     if (errno == 0)
+	       break;
+
+	     if (is_interrupt (errno))
+	       continue;
+	     _pSLerrno_errno = errno;
+	     goto return_error;
+	  }
+
 	name = ep->d_name;
 #  ifdef NEED_D_NAMLEN
 	len = ep->d_namlen;
@@ -1031,8 +1136,9 @@ static void listdir_cmd_wrap (void)
 	     return;
 	  }
 	break;
+
       default:
-	_pSLang_verror (SL_INVALID_PARM, "usage: listdir (string, [opt-string]");
+	_pSLang_verror (SL_Usage_Error, "Usage: listdir (string)");   /* FIXME */
 	return;
      }
 
@@ -1112,6 +1218,9 @@ static SLang_Intrin_Fun_Type PosixDir_Name_Table [] =
 #endif
 #ifdef HAVE_UTIME
    MAKE_INTRINSIC_3("utime", utime_intrin, SLANG_INT_TYPE, SLANG_STRING_TYPE, SLANG_DOUBLE_TYPE, SLANG_DOUBLE_TYPE),
+#endif
+#ifdef HAVE_STATVFS
+   MAKE_INTRINSIC_0("statvfs", statvfs_cmd, SLANG_VOID_TYPE),
 #endif
    SLANG_END_INTRIN_FUN_TABLE
 };
@@ -1181,6 +1290,12 @@ static SLang_IConstant_Type PosixDir_Consts [] =
    MAKE_ICONSTANT("W_OK", W_OK),
    MAKE_ICONSTANT("X_OK", X_OK),
    MAKE_ICONSTANT("F_OK", F_OK),
+#endif
+#ifdef ST_RDONLY
+   MAKE_ICONSTANT("ST_RDONLY", ST_RDONLY),
+#endif
+#ifdef ST_NOSUID
+   MAKE_ICONSTANT("ST_NOSUID", ST_NOSUID),
 #endif
    SLANG_END_ICONST_TABLE
 };
